# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-12 07:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth_token', '0005_auto_20180626_1724'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeviceKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True,
                                        serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('uuid', models.UUIDField(unique=True)),
                ('last_login', models.DateTimeField(blank=True, null=True)),
                ('login_token', models.CharField(max_length=128)),
                ('is_active', models.BooleanField(default=True)),
                ('user_agent',
                 models.CharField(blank=True, max_length=256, null=True)),
                ('user',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                   to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='devicekey',
            unique_together=set([('uuid', 'user')]),
        ),
    ]
