# Generated by Django 2.0 on 2018-06-26 15:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_token', '0004_token_backend'),
    ]

    operations = [
        migrations.AddField(
            model_name='token',
            name='allowed_cookie',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='token',
            name='allowed_header',
            field=models.BooleanField(default=True),
        )
    ]
