# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-13 14:28
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('auth_token', '0006_auto_20190312_0846'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='devicekey',
            options={'verbose_name': 'device key',
                     'verbose_name_plural': 'device keys'},
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True,
                                       verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='last_login',
            field=models.DateTimeField(blank=True, null=True,
                                       verbose_name='last login'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='login_token',
            field=models.CharField(max_length=128, verbose_name='login token'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='user',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='user_agent',
            field=models.CharField(blank=True, max_length=256, null=True,
                                   verbose_name='user agent'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='uuid',
            field=models.UUIDField(unique=True, verbose_name='UUID'),
        ),
    ]
