# Generated by Django 2.0 on 2019-07-23 12:10

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth_token', '0009_auto_20190622_1920'),
    ]

    operations = [
        migrations.AlterField(
            model_name='devicekey',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='last_login',
            field=models.DateTimeField(blank=True, null=True, verbose_name='last login'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL,
                                    verbose_name='user'),
        ),
        migrations.AlterField(
            model_name='devicekey',
            name='uuid',
            field=models.CharField(max_length=32, verbose_name='UUID'),
        ),
    ]
